# dart_parser

This is an API designed to parse dart code and generate an intuitive tree for editing dart source code directly.
It will open a text file (typically a `.dart` file) and parse it according to the dart grammar ([v2.2](https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf)).

**NOTE**:
The format the parser outputs is syntactically the same as as any code parsed, but it will almost certaintly change the format.
It is not recommended to use this this parser to overwrite any files you may have written by hand.

## Getting Started

The project is written in Dart, and can be imported as a standard Dart library.
Thus, simply add it to your `pubspec.yaml` like so:
```
dependencies:
  dart_parser:
    path: [PATH_TO_REPO]
```

Then import it into whatever code you are using by importing:
```
import 'package:source_parser/dart_parser.dart';
```

## Prerequisites

The only prerequisite should be Dart version >=2.2.0

## Example

This lets you parse either a string or a file (sync or async). Here is an example of each:

File async:
```
import 'package:source_parser/dart_parser.dart';

void main() async {
  Parser parser = Parser();
  await parser.parse('src/top_level/Combinator.dart');
  print (parser);
}
```

File sync:
```
import 'package:source_parser/dart_parser.dart';

void main() {
  Parser parser = Parser()..parseSync('src/top_level/Combinator.dart');
  print (parser);
}
```

String (always sync):
```
import 'package:source_parser/dart_parser.dart';

void main() {
  Parser parser = Parser()..parseStr('class Test {}');
  print (parser);
}
```

## License

This project is licensed under the BSD 3-Clause "New" or "Revised" License - see the [LICENSE.md](LICENSE.md) file for details
