import '../ParserExceptions.dart';
import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Literals.dart';

/*
<literal> ::= <nullLiteral>
| <booleanLiteral>
| <numericLiteral>
| <stringLiteral>
| <symbolLiteral>
| <mapLiteral>
| <setLiteral>
| <setOrMapLiteral>
| <listLiteral>

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 81
*/
class Literal extends ParserType {
  int caseNum;
  NullLiteral nullLiteral;
  BooleanLiteral booleanLiteral;
  NumericLiteral numericLiteral;
  StringLiteral stringLiteral;
  SymbolLiteral symbolLiteral;
  MapLiteral mapLiteral;
  SetLiteral setLiteral;
  SetOrMapLiteral setOrMapLiteral;
  ListLiteral listLiteral;

  Literal(Lexer lexer) {
    int attempt, maxAttempts = 9;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
          case 2: _case2(lexer); break;
          case 3: _case3(lexer); break;
          case 4: _case4(lexer); break;
          case 5: _case5(lexer); break;
          case 6: _case6(lexer); break;
          case 7: _case7(lexer); break;
          case 8: _case8(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in Literal: ALL FAILED');

  }
  
  /* <nullLiteral> */
  void _case0(Lexer lexer) {
    nullLiteral = NullLiteral(lexer);
    caseNum = 0;
  }
  
  /* <booleanLiteral> */
  void _case1(Lexer lexer) {
    booleanLiteral = BooleanLiteral(lexer);
    caseNum = 1;
  }

  /* <numericLiteral> */
  void _case2(Lexer lexer) {
    numericLiteral = NumericLiteral(lexer);
    caseNum = 2;
  }

  /* <stringLiteral> */
  void _case3(Lexer lexer) {
    stringLiteral = StringLiteral(lexer);
    caseNum = 3;
  }


  /* <symbolLiteral> */
  void _case4(Lexer lexer) {
    symbolLiteral = SymbolLiteral(lexer);
    caseNum = 4;
  }
  
  /* <mapLiteral> */
  void _case5(Lexer lexer) {
    mapLiteral = MapLiteral(lexer);
    caseNum = 5;
  }
  
  /* <setLiteral> */
  void _case6(Lexer lexer) {
    setLiteral = SetLiteral(lexer);
    caseNum = 6;
  }
  
  /* <setOrMapLiteral> */
  void _case7(Lexer lexer) {
    setOrMapLiteral = SetOrMapLiteral(lexer);
    caseNum = 7;
  }
  
  /* <listLiteral> */
  void _case8(Lexer lexer) {
    listLiteral = ListLiteral(lexer);
    caseNum = 8;
  }

  String toString() {
    switch (caseNum) {
      case 0: return '$nullLiteral'; break;
      case 1: return '$booleanLiteral'; break;
      case 2: return '$numericLiteral'; break;
      case 3: return '$stringLiteral'; break;
      case 4: return '$symbolLiteral'; break;
      case 5: return '$mapLiteral'; break;
      case 6: return '$setLiteral'; break;
      case 7: return '$setOrMapLiteral'; break;
      case 8: return '$listLiteral'; break;
    }
    return '';
  }

}
