import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';
import '../core/Type.dart';
import '../main/MetaData.dart';
import '../sub_types/FormalParameterPart.dart';

/*
<functionSignature> ::= <metadata> <type>? <identifier> <formalParameterPart>

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 17
*/

class FunctionSignature extends ParserType {
  MetaData metaData;
  Type type;
  Identifier identifier;
  FormalParameterPart formalParameterPart;

  FunctionSignature(Lexer lexer) {
    metaData = MetaData(lexer);

    int snapShot = lexer.currentIndex;
    try {
      type = Type(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    identifier = Identifier(lexer);
    formalParameterPart = FormalParameterPart(lexer);
  }
  
  String toString() {
    return '$metaData' + (type != null ? '$type ' : '') + '$identifier$formalParameterPart';
  }

}


